import discord
from discord.ext import tasks, commands

from src.config_dict import ConfigDict

from src.cogs.configuration import Configuration
from src.cogs.notification import Notification
from src.cogs.roles import Roles


class NTPubBot(discord.ext.commands.Bot):
    def __init__(self, config, *args, **options):
        super().__init__(*args, **options)
        self.config = config

    async def on_ready(self):
        print('Logged in as')
        print(self.user.name)
        print(self.user.id)
        print('------')


def main():
    intents = discord.Intents.default()
    intents.members = True

    config = ConfigDict()
    bot = NTPubBot(config, intents=intents, command_prefix="!")

    config_cog = Configuration(bot, bot.config)
    roles_cog = Roles(bot, bot.config["role_id"], bot.config["message_id"],
                      bot.config["channel_id"])
    notification_cog = Notification(bot, bot.config)

    bot.add_cog(config_cog)
    bot.add_cog(roles_cog)
    bot.add_cog(notification_cog)

    bot.run(bot.config["discord_token"])


if __name__ == "__main__":
    main()
