from concurrent.futures.thread import ThreadPoolExecutor

import valve.source
import valve.source.a2s
import valve.source.master_server


def get_server_list():
    with valve.source.master_server.MasterServerQuerier() as msq:
        return [address for address in msq.find(gamedir=u"NeotokyoSource")]


def get_populated_servers(server_list, min_pop):
    with ThreadPoolExecutor() as ex:
        futures = [ex.submit(get_populated_server_info, address, min_pop) for address in server_list]

    results = [f.result() for f in futures]
    # Servers that got returned as None are not pubbing
    return [server for server in results if server is not None]


def get_populated_server_info(address, min_pop):
    try:
        with valve.source.a2s.ServerQuerier(address) as server:
            info = server.info()
            players = server.players()

            # Locked servers are not pubbing, so skip them
            if info["password_protected"] == 1:
                return None

            # AFAIK bot is not included in "player_count" so no need to readjust it man
            if players["player_count"] >= min_pop:
                # Reduce amount of object juggling in nt_bot.py by appending all info to one object
                info["player_count"] = players["player_count"]
                return info

    except valve.source.a2s.NoResponseError:
        return None